@@ -1,13 +1,14 @@
 import argparse
 import json
 import os.path
+import shutil
 from pathlib import Path
+from subprocess import check_call, CalledProcessError
 
-from hypy_utils import printc, write
+from hypy_utils import printc, write, json_stringify
 from hypy_utils.dict_utils import remove_nones
-from hypy_utils.file_utils import escape_filename
 
-from .convert_media_types import tgs_to_apng
+from .convert_media_types import tgs_to_apng, extract_album_art
 from .pyro.consts import HTML
 
 test_text = [
@@ -55,8 +56,8 @@ def convert_text(text: str | list[dict | str] | None) -> str | None:
                     return f"<b>{t}</b>"
                 case "spoiler":
                     return f'<span class="spoiler"><span>{t}</span></span>'
-                # case "link":
-                #     return f'<a href="{t}">{t}</a>'
+                case "link":
+                    return f'<a href="{t}">{t}</a>'
                 # case "mention_name":
                 #     id = e['user_id']
                 #     return f'<a href="https://t.me/{id}">{t}</a>'
@@ -115,26 +116,21 @@ def process_file_path(path: str | None) -> str | None:
     if path.endswith(".tgs"):
         path = str(tgs_to_apng(p / path).relative_to(p))
 
-    url = escape_filename(path)
-    if url == path:
-        return path
-
-    # Move file
-    if os.path.islink(p / url):
-        os.unlink(p / url)
-    os.symlink((p / path).relative_to((p / url).parent), p / url)
-    return url
+    return path
 
 
 def parse_file(d: dict) -> dict | None:
     file = d.get("file")
     if file is None:
         return None
-    return {
+
+    # Create file
+    file = {
         "url": process_file_path(file),
         "thumb": process_file_path(d.get("thumbnail")),
         "mime_type": d.get("mime_type"),
         "size": os.path.getsize(p / file),
+        "original_name": d.get("original_name"),
 
         # Media
         "media_type": d.get("media_type"),
@@ -154,6 +150,19 @@ def parse_file(d: dict) -> dict | None:
         "performer": d.get("performer")
     }
 
+    # Convert image file to photo
+    if not file['media_type'] and file['mime_type'].startswith('image'):
+        file['media_type'] = "photo"
+
+    # Add image for missing album cover art
+    if file['media_type'] == 'audio_file' and not file['thumb']:
+        fp = p / file['url']
+        op = extract_album_art(fp)
+        if op:
+            file['thumb'] = op.relative_to(p)
+
+    return file
+
 
 def get_image(d: dict) -> dict:
     return {"url": d.get("photo"), "width": d.get("width"), "height": d.get("height")}
@@ -224,6 +233,10 @@ def convert_msg(d: dict) -> dict | None:
         # TODO: Add more fields
     }
 
+    # Convert image file group to photo group
+    if msg.get('files') and msg['files'][0]['media_type'] == 'photo':
+        msg['images'] = msg.pop('files')
+
     return remove_nones(msg)
 
 
@@ -279,6 +292,38 @@ groups: dict[int, list[dict]]
 processed_groups: dict[int, int]
 
 
+def convert_original_filenames(json_path: Path):
+    """
+    Convert filenames in the original result.json
+    """
+    renamed: dict[str, str] = {}
+    results = json.loads(json_path.read_text())
+
+    if results.get('processed'):
+        return
+
+    for k in ['file', 'thumbnail']:
+        for r in [r for r in results['messages'] if k in r]:
+            orig = str(r[k])
+            if orig in renamed:
+                r[k] = renamed[orig]
+                continue
+
+            f = p / r[k]
+            new = f.with_name(f"{r['id']}{'_thumb' if k == 'thumbnail' else ''}{f.suffix}")
+            np = str(new.relative_to(p))
+            if new != f:
+                printc(f"&6Renaming &r{orig} &6to &r{np}")
+                shutil.move(f, new)
+                if k == 'file':
+                    r['original_name'] = f.name
+                r[k] = np
+                renamed[orig] = np
+    results['processed'] = True
+
+    write(json_path, json_stringify(results, indent=2))
+
+
 def run():
     global p, id_map, groups, processed_groups
     parser = argparse.ArgumentParser("Telegram export converter",
@@ -290,6 +335,9 @@ def run():
     f = p / "result.json"
     assert f.is_file(), f"Error: File {f} not found"
 
+    # Convert file names to url-safe file names
+    convert_original_filenames(f)
+
     # Read export result json
     j: list[dict] = json.loads(f.read_text())["messages"]
     id_map = {d['id']: d for d in j}
@@ -309,8 +357,8 @@ def run():
     j = [convert_msg(d) for d in j]
     j = [d for d in j if d is not None]
 
-    (p / "posts.json").write_text(json.dumps(j, indent=2, ensure_ascii=False))
-    write(p / "index.html", HTML)
+    write(p / "posts.json", json_stringify(j, indent=2))
+    write(p / "index.html", HTML.replace("$$POSTS_DATA$$", json_stringify(j)))
 
     printc(f"&aDone! Saved to {p / 'posts.json'}")
 
